import { getNextSizeRelatedInfo } from './utils';
function getBarID(indexOfBar) {
    return indexOfBar * 2 + 1;
}
function getSectionID(indexOfSection) {
    return indexOfSection * 2;
}
export class Resizer {
    constructor(resizeResult) {
        this.resizeResult = resizeResult;
        this.isDiscarded = false;
    }
    resizeSection(indexOfSection, config) {
        if (this.isDiscarded) {
            return;
        }
        const sectionID = getSectionID(indexOfSection);
        const currentSize = this.getSize(sectionID);
        if (currentSize >= 0 && config.toSize >= 0) {
            const offset = config.toSize - currentSize;
            if (sectionID === this.resizeResult.sizeInfoArray.length - 1 ||
                config.preferMoveLeftBar) {
                this.moveBar(indexOfSection - 1, { withOffset: -offset });
            }
            else {
                this.moveBar(indexOfSection, { withOffset: offset });
            }
        }
    }
    moveBar(indexOfBar, config) {
        if (this.isDiscarded) {
            return;
        }
        this.resizeResult = getNextSizeRelatedInfo(getBarID(indexOfBar), config.withOffset, this.resizeResult.sizeInfoArray);
    }
    discard() {
        this.isDiscarded = true;
    }
    isSectionResized(indexOfSection) {
        const sectionID = getSectionID(indexOfSection);
        if ('defaultSizeInfoArray' in this.resizeResult) {
            return (this.getSize(sectionID) !==
                this.resizeResult.defaultSizeInfoArray[sectionID].currentSize);
        }
        else {
            return false;
        }
    }
    isBarActivated(indexOfBar) {
        if ('barID' in this.resizeResult) {
            return this.resizeResult.barID === getBarID(indexOfBar);
        }
        else {
            return false;
        }
    }
    getSectionSize(indexOfSection) {
        return this.getSize(getSectionID(indexOfSection));
    }
    getResult() {
        return { ...this.resizeResult, discard: this.isDiscarded };
    }
    getTotalSize() {
        return this.resizeResult.sizeInfoArray
            .filter((sizeInfo, index) => sizeInfo && index % 2 === 0)
            .reduce((total, { currentSize }) => total + currentSize, 0);
    }
    getSize(index) {
        const sizeInfo = this.resizeResult.sizeInfoArray[index];
        return sizeInfo ? sizeInfo.currentSize : -1;
    }
}
