import { scan } from 'rxjs/operators';
import { BarActionType, } from '../types';
import { DEFAULT_COORDINATE_OFFSET, getNextSizeRelatedInfo } from './utils';
const DEFAULT_BAR_ACTION_SCAN_RESULT = {
    barID: -1,
    offset: 0,
    type: BarActionType.DEACTIVATE,
    originalCoordinate: DEFAULT_COORDINATE_OFFSET,
    defaultSizeInfoArray: [],
    sizeInfoArray: [],
    discard: true,
    flexGrowRatio: 0,
};
export function scanBarAction(config) {
    return scan((prevResult, action) => {
        const result = {
            barID: action.barID,
            type: action.type,
        };
        switch (action.type) {
            case BarActionType.ACTIVATE:
                const { sizeInfoArray, flexGrowRatio } = config.getSizeRelatedInfo();
                return {
                    ...DEFAULT_BAR_ACTION_SCAN_RESULT,
                    ...result,
                    originalCoordinate: action.coordinate,
                    defaultSizeInfoArray: sizeInfoArray,
                    sizeInfoArray,
                    flexGrowRatio,
                };
            case BarActionType.MOVE:
                const offset = config.calculateOffset(action.coordinate, prevResult.originalCoordinate);
                return {
                    ...result,
                    ...getNextSizeRelatedInfo(action.barID, offset, prevResult.defaultSizeInfoArray),
                    offset,
                    originalCoordinate: prevResult.originalCoordinate,
                    defaultSizeInfoArray: prevResult.defaultSizeInfoArray,
                    discard: false,
                };
            case BarActionType.DEACTIVATE:
                return DEFAULT_BAR_ACTION_SCAN_RESULT;
        }
    }, DEFAULT_BAR_ACTION_SCAN_RESULT);
}
