import * as React from 'react';
import { Subscription } from 'rxjs';
import { filter, map, tap } from 'rxjs/operators';
import { withResizerContext } from '../context';
import { isValidNumber } from '../utils';
import { StyledSection } from './Section.styled';
class SectionComponent extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.defaultInnerRef = React.createRef();
        this.id = this.props.context.createID(this.props);
        this.subscription = new Subscription();
        this.flexGrowRatio = 0;
        this.resize$ = this.props.context.sizeRelatedInfo$.pipe(map(({ sizeInfoArray, flexGrowRatio }) => ({
            sizeInfo: sizeInfoArray[this.id],
            flexGrowRatio,
        })), filter(({ sizeInfo }) => !!sizeInfo), tap(({ sizeInfo, flexGrowRatio }) => {
            this.sizeInfo = sizeInfo;
            this.flexGrowRatio = flexGrowRatio;
            if (this.ref.current) {
                const { flexGrow, flexShrink, flexBasis } = this.getStyle(sizeInfo, flexGrowRatio);
                this.ref.current.style.flexGrow = `${flexGrow}`;
                this.ref.current.style.flexShrink = `${flexShrink}`;
                this.ref.current.style.flexBasis = `${flexBasis}px`;
                this.onSizeChanged(sizeInfo.currentSize);
            }
        }));
    }
    get ref() {
        return this.props.innerRef || this.defaultInnerRef;
    }
    componentDidMount() {
        this.subscription.add(this.resize$.subscribe());
        this.props.context.populateInstance(this.id, this.ref);
    }
    componentWillUnmount() {
        this.subscription.unsubscribe();
    }
    render() {
        return (React.createElement(StyledSection, Object.assign({}, this.props, this.getStyle(), { ref: this.ref })));
    }
    onSizeChanged(currentSize) {
        if (typeof this.props.onSizeChanged === 'function') {
            this.props.onSizeChanged(currentSize);
        }
    }
    getFlexShrink() {
        if (isValidNumber(this.props.size)) {
            return 0;
        }
        else {
            return this.props.disableResponsive ? 1 : 0;
        }
    }
    getStyle(sizeInfo = this.sizeInfo, flexGrowRatio = this.flexGrowRatio) {
        const flexShrink = this.getFlexShrink();
        if (sizeInfo) {
            const { disableResponsive, currentSize } = sizeInfo;
            return {
                flexShrink,
                flexGrow: disableResponsive ? 0 : flexGrowRatio * currentSize,
                flexBasis: disableResponsive ? currentSize : 0,
            };
        }
        else {
            const size = this.props.size || this.props.defaultSize;
            if (isValidNumber(size)) {
                return {
                    flexShrink,
                    flexGrow: 0,
                    flexBasis: size,
                };
            }
            else {
                return {
                    flexShrink,
                    flexGrow: 1,
                    flexBasis: 0,
                };
            }
        }
    }
}
export const Section = withResizerContext(SectionComponent);
